/** -*-c++-*-
 * Author: Antonio Cisternino
 * Last Update: 22/05/2001
 * Version: 0.5
 */

// Constants
var shell__PROG   = "ObjShell";
var shell__COPY   = "(C) 2001";
var shell__AUTH   = "Antonio Cisternino, Dario Maggiorini";
var shell__DEPT   = "University of Pisa and Milano";
var shell__REV    = "v.0.5";

// Global variables
var stdin, stdout, fso, conf;
var console = false;
var wshsh = false;
var homeDirectory = homeDir();
var history = new historyobj();
var shell__currReader = "Object";
var shell__descs = new Descriptions();
var shell__modes = new Modes();
var shell__numin = 0;
var shell__vars  = new Variables();

// Read command line switches
conf = parseSwitches();

if (conf.console) {
  // Used to show the console
  console = CreateObject("OConsole.Console");
  console.Show();
}

fso = CreateObject("Scripting.FileSystemObject");

stdin = fso.OpenTextFile("CONIN$", 1);
stdout = fso.OpenTextFile("CONOUT$", 2);

// Load utils...
eval(fimport(homeDirectory + "lib\\shellUtils.js"));

// Load known reader
eval(rimport(homeDirectory + "lib\\ObjectReader.rd"));
eval(rimport(homeDirectory + "lib\\CommandReader.rd"));
eval(rimport(homeDirectory + "lib\\PAccountReader.rd"));
eval(rimport(homeDirectory + "lib\\DirectoryReader.rd"));

stdout.write(shell__PROG+" "+shell__REV+"\n"+shell__AUTH+"\n"+shell__DEPT+"\n"+shell__COPY+"\n\n");

// Shell Fetch-Execute
while (true) {
  // Prompt  
  stdout.write(shell__currReader + ": "+shell__numin+"> ");

  var shell__cmd = resolveNS(eval(resolveNS("::" + shell__currReader + 
				     "::Reader()")));

  if (shell__cmd == "exit")
    break;
  else if (shell__cmd == "")
    continue;
  else
    try {
      stdout.write(eval(shell__cmd) + "\n");
      // Add the command to the history if succeded
      history.add(shell__cmd);
      ++shell__numin;
    } catch (e) {
      cmdBuf = "";
      stdout.write("Error number " + e.number + ": " + e.description + "\n");
      //throw e;
    }
}

//////////////////////////////////////////////////////////////////////
// Objects
//////////////////////////////////////////////////////////////////////
function Variables () {
  this.length = 0;
}

function Parameters() {
  this.console = true;
}

function Descriptions () {
}

function Modes() {
  this.length = 0;
}

function historyobj(a) {
  this.active = (a) ? true : false;
  this.length = 0;
  this.print = shell__historyobj_print;
  this.remove = shell__historyobj_delete;
  this.add = shell__historyobj_add;
  this.saveTo = shell__historyobj_saveTo;
}

//////////////////////////////////////////////////////////////////////
// Methods
//////////////////////////////////////////////////////////////////////
function shell__historyobj_print(num) {
  var i, ret = "";
  for (i = 0; i < this.length; i++) {
    if (!this[i])
      continue;

    if (num)
      ret += i + ": ";
    ret += this[i] + "\n";
  }

  return ret;
}

function shell__historyobj_delete(num) {
  if ((num >= 0) && (num < this.length))
    this[num] = false;
  return "";
}

function shell__historyobj_add(cmd) {
  if (this.active && !(/^\s*history\./).test(cmd))
    this[this.length++] = cmd;
}

function shell__historyobj_saveTo(fn) {
  var f = fso.CreateTextFile(fn);
  f.Write(this.print());
  f.Close();
}

//////////////////////////////////////////////////////////////////////
// Functions
//////////////////////////////////////////////////////////////////////

function homeDir() {
  return (WScript.ScriptFullName).replace(new RegExp(WScript.ScriptName + "$"), "");
}

function trim(str) {
  var re = /^\s*(.*?)\s*$/;
  re.exec(str);
  return RegExp.$1;
}

function setDescription(sym, txt) {
  shell__descs[sym] = txt;
}

function parseSwitches() {
  var ret = new Parameters();
  var item;
  var i = 0;

  while (i < WScript.Arguments.Count()) {
    item = WScript.Arguments.item(i);

    if (item == "-C")
      ret.console = false;

    ++i;
  }

  return ret;
}

function addMode(name, description) {
  shell__modes[shell__modes.length++] = name;
  shell__modes[name] = description;
}

function fimport(file) {
  var  f = fso.OpenTextFile(file, 1);  
  var ret = f.ReadAll();
  f.Close();

  return ret;
}

function resolveNS(str) {
  return str.replace(/::/g, "__");
}

function rimport(file) {
  var ret = fimport(file);

  var nm = "voidns";

  if ((/namespace\s+([^\;]+)\;/m).test(ret)) {
    nm = RegExp.$1;

    ret = ret.replace(/namespace\s+[^\;]+\;/, "");
  }

  ret = ret.replace(/function\s+([^\(\s]+)\s*\(/g, "function ::"+nm+"::$1(");

  ret = ret.replace(/@@/g, "::"+nm+"::");

  return resolveNS(ret);
}

function CreateObject(obj) {
  return WScript.CreateObject(obj);
}

function setMode(name) {
  setDescription("setMode", "Change the current mode");

  var i = 0;

  while (i < shell__modes.length) {
    if (name == shell__modes[i]) {
      shell__currReader = name;
      return "";
    }
    i++;
  }

  return "Mode called " + mode + " not found!\n";
}


function listModes() {
  setDescription("listModes", "Show a list of loaded Modes");

  stdout.write("List of available modes:\n\n");
  
  var i = 0;

  for (i = 0; i < shell__modes.length; i++) {
    stdout.write(shell__modes[i] + "\t" + shell__modes[shell__modes[i]]+ "\n");
  }

  return "\nDone.";  
}

function source(funcname) {
  if (eval("typeof "+funcname) == "function")
    return eval(funcname);

  return funcname + " is not a function!";
}

function describeCOMObject(sym) {
  return "PROPERTIES:\n\n" + console.ListProperties(sym) + "\nMETHODS:\n\n" + console.ListMethods(sym);
}

function describe(sym) {
  setDescription("describe", "describe(sym)\n\nReturn the description of the object called sym");

  if (typeof(sym) == "object")
    return describeCOMObject(sym);

  if (shell__descs[sym])
    return shell__descs[sym];

  return "Description for "+sym+" not found";
}

function help(obj, id) {
  return console.ShowHelp(obj, id);
}

function ShellReadLine() {
  var line = "";
  var ch;

  while (!console.CheckForTab()) {
    WScript.Echo(ch = stdin.Read(1));
    if (ch == "\n")
      break;
    line += ch;
  }  

  return line;
}
